package com.iconplus.smartproc.service.manajemenuser;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.Users;
import com.iconplus.smartproc.model.request.UsersRequest;

import com.iconplus.smartproc.model.response.UsersResponse;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.service.CommonService;
import com.iconplus.smartproc.util.Constants;
import com.iconplus.smartproc.util.EncryptDecryptUtils;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class PostCreateUserService implements BaseService<UsersRequest, UsersResponse> {

    private UsersRepository usersRepository;
    private CommonService commonService;

    public PostCreateUserService(UsersRepository usersRepository,
                                CommonService commonService) {
        this.usersRepository = usersRepository;
       this.commonService = commonService;
    }

    @Override
    public UsersResponse execute(UsersRequest input) {

        var users = usersRepository.findByEmailAndIsDeleteFalse(input.getEmail());
        if (users.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "User", input.getEmail()));
        }

        commonService.validateUserRequst(input);

        String password = commonService.getPassword(input.getPassword());

        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        String encryptedPassword = encoder.encode(password);
        input.setPassword(encryptedPassword);

        Users usersEntity = Users.builder()
                .instansiId(input.getInstansiId())
                .bidangId(input.getBidangId())
                .nama(input.getNama())
                .username(input.getUsername())
                .jabatanId(input.getJabatanId())
                .roleId(input.getRoleId())
                .email(input.getEmail())
                .password(input.getPassword())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();

        var result = usersRepository.save(usersEntity);
        return UsersResponse.builder()
                .id(result.getId())
                .build();

    }
}
